/**
 * Copyright (c) 2000-2006 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.service.persistence;

import com.liferay.portal.NoSuchDataTrackerException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.model.DataTracker;
import com.liferay.portal.model.impl.DataTrackerImpl;
import com.liferay.portal.service.persistence.BasePersistence;

import com.liferay.util.dao.hibernate.QueryUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import java.util.Iterator;
import java.util.List;

/**
 * <a href="DataTrackerPersistence.java.html"><b><i>View Source</i></b></a>
 *
 * @author  Brian Wing Shun Chan
 *
 */
public class DataTrackerPersistence extends BasePersistence {
	public DataTracker create(String dataTrackerId) {
		DataTracker dataTracker = new DataTrackerImpl();
		dataTracker.setNew(true);
		dataTracker.setPrimaryKey(dataTrackerId);

		return dataTracker;
	}

	public DataTracker remove(String dataTrackerId)
		throws NoSuchDataTrackerException, SystemException {
		Session session = null;

		try {
			session = openSession();

			DataTracker dataTracker = (DataTracker)session.get(DataTrackerImpl.class,
					dataTrackerId);

			if (dataTracker == null) {
				if (_log.isWarnEnabled()) {
					_log.warn("No DataTracker exists with the primary key " +
						dataTrackerId);
				}

				throw new NoSuchDataTrackerException(
					"No DataTracker exists with the primary key " +
					dataTrackerId);
			}

			return remove(dataTracker);
		}
		catch (HibernateException he) {
			throw new SystemException(he);
		}
		finally {
			closeSession(session);
		}
	}

	public DataTracker remove(DataTracker dataTracker)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();
			session.delete(dataTracker);
			session.flush();

			return dataTracker;
		}
		catch (HibernateException he) {
			throw new SystemException(he);
		}
		finally {
			closeSession(session);
		}
	}

	public com.liferay.portal.model.DataTracker update(
		com.liferay.portal.model.DataTracker dataTracker)
		throws SystemException {
		return update(dataTracker, false);
	}

	public com.liferay.portal.model.DataTracker update(
		com.liferay.portal.model.DataTracker dataTracker, boolean saveOrUpdate)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			if (saveOrUpdate) {
				session.saveOrUpdate(dataTracker);
			}
			else {
				if (dataTracker.isNew()) {
					session.save(dataTracker);
				}
			}

			session.flush();
			dataTracker.setNew(false);

			return dataTracker;
		}
		catch (HibernateException he) {
			throw new SystemException(he);
		}
		finally {
			closeSession(session);
		}
	}

	public DataTracker findByPrimaryKey(String dataTrackerId)
		throws NoSuchDataTrackerException, SystemException {
		DataTracker dataTracker = fetchByPrimaryKey(dataTrackerId);

		if (dataTracker == null) {
			if (_log.isWarnEnabled()) {
				_log.warn("No DataTracker exists with the primary key " +
					dataTrackerId);
			}

			throw new NoSuchDataTrackerException(
				"No DataTracker exists with the primary key " + dataTrackerId);
		}

		return dataTracker;
	}

	public DataTracker fetchByPrimaryKey(String dataTrackerId)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			return (DataTracker)session.get(DataTrackerImpl.class, dataTrackerId);
		}
		catch (HibernateException he) {
			throw new SystemException(he);
		}
		finally {
			closeSession(session);
		}
	}

	public List findAll() throws SystemException {
		return findAll(QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}

	public List findAll(int begin, int end) throws SystemException {
		return findAll(begin, end, null);
	}

	public List findAll(int begin, int end, OrderByComparator obc)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			StringBuffer query = new StringBuffer();
			query.append("FROM com.liferay.portal.model.DataTracker ");

			if (obc != null) {
				query.append("ORDER BY " + obc.getOrderBy());
			}

			Query q = session.createQuery(query.toString());
			q.setCacheable(true);

			return QueryUtil.list(q, getDialect(), begin, end);
		}
		catch (HibernateException he) {
			throw new SystemException(he);
		}
		finally {
			closeSession(session);
		}
	}

	public void removeAll() throws SystemException {
		Iterator itr = findAll().iterator();

		while (itr.hasNext()) {
			remove((DataTracker)itr.next());
		}
	}

	public int countAll() throws SystemException {
		Session session = null;

		try {
			session = openSession();

			StringBuffer query = new StringBuffer();
			query.append("SELECT COUNT(*) ");
			query.append("FROM com.liferay.portal.model.DataTracker");

			Query q = session.createQuery(query.toString());
			q.setCacheable(true);

			Iterator itr = q.list().iterator();

			if (itr.hasNext()) {
				Long count = (Long)itr.next();

				if (count != null) {
					return count.intValue();
				}
			}

			return 0;
		}
		catch (HibernateException he) {
			throw new SystemException(he);
		}
		finally {
			closeSession(session);
		}
	}

	protected void initDao() {
	}

	private static Log _log = LogFactory.getLog(DataTrackerPersistence.class);
}