/**
 * Copyright (c) 2000-2006 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.service.persistence;

import com.liferay.portal.model.ModelListener;
import com.liferay.portal.util.PropsUtil;

import com.liferay.util.GetterUtil;
import com.liferay.util.Validator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <a href="UserIdMapperUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author  Brian Wing Shun Chan
 *
 */
public class UserIdMapperUtil {
	public static com.liferay.portal.model.UserIdMapper create(
		com.liferay.portal.service.persistence.UserIdMapperPK userIdMapperPK) {
		return getPersistence().create(userIdMapperPK);
	}

	public static com.liferay.portal.model.UserIdMapper remove(
		com.liferay.portal.service.persistence.UserIdMapperPK userIdMapperPK)
		throws com.liferay.portal.SystemException, 
			com.liferay.portal.NoSuchUserIdMapperException {
		ModelListener listener = _getListener();

		if (listener != null) {
			listener.onBeforeRemove(findByPrimaryKey(userIdMapperPK));
		}

		com.liferay.portal.model.UserIdMapper userIdMapper = getPersistence()
																 .remove(userIdMapperPK);

		if (listener != null) {
			listener.onAfterRemove(userIdMapper);
		}

		return userIdMapper;
	}

	public static com.liferay.portal.model.UserIdMapper remove(
		com.liferay.portal.model.UserIdMapper userIdMapper)
		throws com.liferay.portal.SystemException {
		ModelListener listener = _getListener();

		if (listener != null) {
			listener.onBeforeRemove(userIdMapper);
		}

		userIdMapper = getPersistence().remove(userIdMapper);

		if (listener != null) {
			listener.onAfterRemove(userIdMapper);
		}

		return userIdMapper;
	}

	public static com.liferay.portal.model.UserIdMapper update(
		com.liferay.portal.model.UserIdMapper userIdMapper)
		throws com.liferay.portal.SystemException {
		ModelListener listener = _getListener();
		boolean isNew = userIdMapper.isNew();

		if (listener != null) {
			if (isNew) {
				listener.onBeforeCreate(userIdMapper);
			}
			else {
				listener.onBeforeUpdate(userIdMapper);
			}
		}

		userIdMapper = getPersistence().update(userIdMapper);

		if (listener != null) {
			if (isNew) {
				listener.onAfterCreate(userIdMapper);
			}
			else {
				listener.onAfterUpdate(userIdMapper);
			}
		}

		return userIdMapper;
	}

	public static com.liferay.portal.model.UserIdMapper update(
		com.liferay.portal.model.UserIdMapper userIdMapper, boolean saveOrUpdate)
		throws com.liferay.portal.SystemException {
		ModelListener listener = _getListener();
		boolean isNew = userIdMapper.isNew();

		if (listener != null) {
			if (isNew) {
				listener.onBeforeCreate(userIdMapper);
			}
			else {
				listener.onBeforeUpdate(userIdMapper);
			}
		}

		userIdMapper = getPersistence().update(userIdMapper, saveOrUpdate);

		if (listener != null) {
			if (isNew) {
				listener.onAfterCreate(userIdMapper);
			}
			else {
				listener.onAfterUpdate(userIdMapper);
			}
		}

		return userIdMapper;
	}

	public static com.liferay.portal.model.UserIdMapper findByPrimaryKey(
		com.liferay.portal.service.persistence.UserIdMapperPK userIdMapperPK)
		throws com.liferay.portal.SystemException, 
			com.liferay.portal.NoSuchUserIdMapperException {
		return getPersistence().findByPrimaryKey(userIdMapperPK);
	}

	public static com.liferay.portal.model.UserIdMapper fetchByPrimaryKey(
		com.liferay.portal.service.persistence.UserIdMapperPK userIdMapperPK)
		throws com.liferay.portal.SystemException {
		return getPersistence().fetchByPrimaryKey(userIdMapperPK);
	}

	public static java.util.List findByUserId(java.lang.String userId)
		throws com.liferay.portal.SystemException {
		return getPersistence().findByUserId(userId);
	}

	public static java.util.List findByUserId(java.lang.String userId,
		int begin, int end) throws com.liferay.portal.SystemException {
		return getPersistence().findByUserId(userId, begin, end);
	}

	public static java.util.List findByUserId(java.lang.String userId,
		int begin, int end, com.liferay.portal.kernel.util.OrderByComparator obc)
		throws com.liferay.portal.SystemException {
		return getPersistence().findByUserId(userId, begin, end, obc);
	}

	public static com.liferay.portal.model.UserIdMapper findByUserId_First(
		java.lang.String userId,
		com.liferay.portal.kernel.util.OrderByComparator obc)
		throws com.liferay.portal.SystemException, 
			com.liferay.portal.NoSuchUserIdMapperException {
		return getPersistence().findByUserId_First(userId, obc);
	}

	public static com.liferay.portal.model.UserIdMapper findByUserId_Last(
		java.lang.String userId,
		com.liferay.portal.kernel.util.OrderByComparator obc)
		throws com.liferay.portal.SystemException, 
			com.liferay.portal.NoSuchUserIdMapperException {
		return getPersistence().findByUserId_Last(userId, obc);
	}

	public static com.liferay.portal.model.UserIdMapper[] findByUserId_PrevAndNext(
		com.liferay.portal.service.persistence.UserIdMapperPK userIdMapperPK,
		java.lang.String userId,
		com.liferay.portal.kernel.util.OrderByComparator obc)
		throws com.liferay.portal.SystemException, 
			com.liferay.portal.NoSuchUserIdMapperException {
		return getPersistence().findByUserId_PrevAndNext(userIdMapperPK,
			userId, obc);
	}

	public static java.util.List findAll()
		throws com.liferay.portal.SystemException {
		return getPersistence().findAll();
	}

	public static java.util.List findAll(int begin, int end)
		throws com.liferay.portal.SystemException {
		return getPersistence().findAll(begin, end);
	}

	public static java.util.List findAll(int begin, int end,
		com.liferay.portal.kernel.util.OrderByComparator obc)
		throws com.liferay.portal.SystemException {
		return getPersistence().findAll(begin, end, obc);
	}

	public static void removeByUserId(java.lang.String userId)
		throws com.liferay.portal.SystemException {
		getPersistence().removeByUserId(userId);
	}

	public static void removeAll() throws com.liferay.portal.SystemException {
		getPersistence().removeAll();
	}

	public static int countByUserId(java.lang.String userId)
		throws com.liferay.portal.SystemException {
		return getPersistence().countByUserId(userId);
	}

	public static int countAll() throws com.liferay.portal.SystemException {
		return getPersistence().countAll();
	}

	public static void initDao() {
		getPersistence().initDao();
	}

	public static UserIdMapperPersistence getPersistence() {
		return _getUtil()._persistence;
	}

	public void setPersistence(UserIdMapperPersistence persistence) {
		_persistence = persistence;
	}

	private static UserIdMapperUtil _getUtil() {
		if (_util == null) {
			_util = (UserIdMapperUtil)com.liferay.portal.kernel.bean.BeanLocatorUtil.locate(_UTIL);
		}

		return _util;
	}

	private static ModelListener _getListener() {
		if (Validator.isNotNull(_LISTENER)) {
			try {
				return (ModelListener)Class.forName(_LISTENER).newInstance();
			}
			catch (Exception e) {
				_log.error(e);
			}
		}

		return null;
	}

	private static final String _UTIL = UserIdMapperUtil.class.getName();
	private static final String _LISTENER = GetterUtil.getString(PropsUtil.get(
				"value.object.listener.com.liferay.portal.model.UserIdMapper"));
	private static Log _log = LogFactory.getLog(UserIdMapperUtil.class);
	private static UserIdMapperUtil _util;
	private UserIdMapperPersistence _persistence;
}