/**
 * Copyright (c) 2000-2006 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.messageboards.action;

import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.Company;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.Constants;
import com.liferay.portal.util.WebKeys;
import com.liferay.portlet.messageboards.service.MBMessageLocalServiceUtil;
import com.liferay.util.Http;
import com.liferay.util.ParamUtil;
import com.liferay.util.RSSUtil;
import com.liferay.util.Validator;
import com.liferay.util.dao.search.SearchContainer;
import com.liferay.util.servlet.ServletResponseUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.PageContext;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * <a href="RSSAction.java.html"><b><i>View Source</i></b></a>
 *
 * @author  Brian Wing Shun Chan
 *
 */
public class RSSAction extends Action {

	public ActionForward execute(
			ActionMapping mapping, ActionForm form, HttpServletRequest req,
			HttpServletResponse res)
		throws Exception {

		try {
			ServletResponseUtil.sendFile(
				res, null, getRSS(req), Constants.TEXT_XML);

			return null;
		}
		catch (Exception e) {
			req.setAttribute(PageContext.EXCEPTION, e);

			return mapping.findForward(Constants.COMMON_ERROR);
		}
	}

	protected byte[] getRSS(HttpServletRequest req) throws Exception {
		ThemeDisplay themeDisplay =
			(ThemeDisplay)req.getAttribute(WebKeys.THEME_DISPLAY);

		Company company = themeDisplay.getCompany();

		String plid = ParamUtil.getString(req, "p_l_id");
		String categoryId = ParamUtil.getString(req, "categoryId");
		String threadId = ParamUtil.getString(req, "threadId");
		String type = ParamUtil.getString(req, "type", RSSUtil.DEFAULT_TYPE);
		double version = ParamUtil.getDouble(
			req, "version", RSSUtil.DEFAULT_VERSION);

		String entryURL =
			Http.getProtocol(req) + "://" + company.getPortalURL() +
				themeDisplay.getPathMain() +
					"/message_boards/find_message?p_l_id=" + plid +
						"&categoryId=" + categoryId;

		String rss = StringPool.BLANK;

		if (Validator.isNotNull(categoryId)) {
			String feedURL =
				Http.getProtocol(req) + "://" + company.getPortalURL() +
					themeDisplay.getPathMain() +
						"/message_boards/find_category?p_l_id=" + plid +
							"&categoryId=" + categoryId;

			rss = MBMessageLocalServiceUtil.getCategoryMessagesRSS(
				categoryId, 0, SearchContainer.DEFAULT_DELTA, type, version,
				feedURL, entryURL);
		}
		else {
			String feedURL =
				Http.getProtocol(req) + "://" + company.getPortalURL() +
					themeDisplay.getPathMain() +
						"/message_boards/find_thread?p_l_id=" + plid +
							"&threadId=" + threadId;

			rss = MBMessageLocalServiceUtil.getThreadMessagesRSS(
				threadId, 0, SearchContainer.DEFAULT_DELTA, type, version,
				feedURL, entryURL);
		}

		return rss.getBytes();
	}

}