/**
 * Copyright (c) 2000-2006 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.workflow.service.impl;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.service.impl.PrincipalBean;
import com.liferay.portlet.workflow.jbi.WorkflowXMLUtil;
import com.liferay.portlet.workflow.model.WorkflowInstance;
import com.liferay.portlet.workflow.service.WorkflowComponentServiceUtil;
import com.liferay.portlet.workflow.service.WorkflowInstanceService;

import java.rmi.RemoteException;

import java.text.ParseException;

import org.dom4j.DocumentException;

/**
 * <a href="WorkflowInstanceServiceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author  Brian Wing Shun Chan
 *
 */
public class WorkflowInstanceServiceImpl
	extends PrincipalBean implements WorkflowInstanceService {

	public WorkflowInstance addInstance(long definitionId)
		throws PortalException, SystemException {

		try {
			String xml = WorkflowComponentServiceUtil.startWorkflow(
				definitionId);

			return WorkflowXMLUtil.parseInstance(xml);
		}
		catch (DocumentException de) {
			throw new SystemException(de);
		}
		catch (RemoteException re) {
			throw new SystemException(re);
		}
		catch (ParseException pe) {
			throw new SystemException(pe);
		}
	}

	public void signalInstance(long instanceId)
		throws PortalException, SystemException {

		try {
			WorkflowComponentServiceUtil.signalInstance(instanceId);
		}
		catch (RemoteException re) {
			throw new SystemException(re);
		}
	}

	public void signalToken(long instanceId, long tokenId)
		throws PortalException, SystemException {

		try {
			WorkflowComponentServiceUtil.signalToken(instanceId, tokenId);
		}
		catch (RemoteException re) {
			throw new SystemException(re);
		}
	}

}